@@ -11,10 +11,7 @@ from keras_cv_attention_models.attention_layers import (
     drop_connect_rates_split,
     se_module,
     eca_module,
-    deep_stem,
-    quad_stem,
-    tiered_stem,
-    output_block,
+    # output_block,
     add_pre_post_process,
 )
 from keras_cv_attention_models import attention_layers
@@ -41,6 +38,9 @@ def attn_block(
     groups=1,
     group_size=0,
     bn_after_attn=True,
+    use_evo_norm=False,
+    evo_norm_group_size=-1,
+    epsilon=1e-5,
     activation="relu",
     name="",
 ):
@@ -78,7 +78,8 @@ def attn_block(
         nn = keras.layers.AveragePooling2D(pool_size=2, strides=strides, name=name + "pool")(nn)
 
     if bn_after_attn:
-        nn = batchnorm_with_activation(nn, activation=activation, zero_gamma=False, name=name)
+        bn_params = {"use_evo_norm": use_evo_norm, "evo_norm_group_size": evo_norm_group_size, "epsilon": epsilon}
+        nn = batchnorm_with_activation(nn, activation, zero_gamma=False, **bn_params, name=name)
 
     if attn_type is None and se_ratio:
         nn = se_module(nn, se_ratio=se_ratio, divisor=se_divisor, activation=attn_act, name=name + "se_")
@@ -88,7 +89,7 @@ def attn_block(
     return nn
 
 
-def conv_shortcut_branch(inputs, filters, preact=False, strides=1, shortcut_type="conv", activation=None, name=""):
+def conv_shortcut_branch(inputs, filters, preact=False, strides=1, shortcut_type="conv", bn_act_params={}, name=""):
     if shortcut_type is None:
         return None
 
@@ -102,12 +103,12 @@ def conv_shortcut_branch(inputs, filters, preact=False, strides=1, shortcut_type
         shortcut = inputs
     shortcut = conv2d_no_bias(shortcut, filters, 1, strides=strides, name=name + "shortcut_")
     if not preact:  # ResNet
-        shortcut = batchnorm_with_activation(shortcut, activation=activation, zero_gamma=False, name=name + "shortcut_")
+        shortcut = batchnorm_with_activation(shortcut, zero_gamma=False, **bn_act_params, name=name + "shortcut_")
     return shortcut
 
 
 def deep_branch(
-    inputs, filters, strides=1, hidden_channel_ratio=0.25, use_3x3_kernel=False, bn_after_attn=True, activation="relu", attn_block_params={}, name=""
+    inputs, filters, strides=1, hidden_channel_ratio=0.25, use_3x3_kernel=False, bn_after_attn=True, bn_act_params={}, attn_block_params={}, name=""
 ):
     hidden_filter = int(filters * hidden_channel_ratio)
     if use_3x3_kernel:
@@ -116,9 +117,9 @@ def deep_branch(
         # strides = 1
     else:
         nn = conv2d_no_bias(inputs, hidden_filter, 1, strides=1, padding="VALID", name=name + "deep_1_")
-    nn = batchnorm_with_activation(nn, activation=activation, zero_gamma=False, name=name + "deep_1_")
+    nn = batchnorm_with_activation(nn, zero_gamma=False, **bn_act_params, name=name + "deep_1_")
     # bn_after_attn = False if use_3x3_kernel else bn_after_attn
-    nn = attn_block(nn, hidden_filter, strides, **attn_block_params, bn_after_attn=bn_after_attn, activation=activation, name=name + "deep_2_")
+    nn = attn_block(nn, hidden_filter, strides, **attn_block_params, **bn_act_params, bn_after_attn=bn_after_attn, name=name + "deep_2_")
 
     if not use_3x3_kernel:
         nn = conv2d_no_bias(nn, filters, 1, strides=1, padding="VALID", name=name + "deep_3_")
@@ -137,6 +138,9 @@ def aot_block(
     bn_after_attn=True,
     shortcut_type="conv",
     use_block_output_activation=True,
+    use_evo_norm=False,
+    epsilon=1e-5,
+    evo_norm_group_size=-1,
     activation="relu",
     attn_block_params={},
     name="",
@@ -151,18 +155,20 @@ def aot_block(
             # print(f">>>> Halo pad: {inputs.shape = }, {pad_head_h = }, {pad_tail_h = }, {pad_head_w = }, {pad_tail_w = }")
             inputs = keras.layers.ZeroPadding2D(padding=((pad_head_h, pad_tail_h), (pad_head_w, pad_tail_w)), name=name + "gap_pad")(inputs)
 
+    bn_params = {"use_evo_norm": use_evo_norm, "evo_norm_group_size": evo_norm_group_size, "epsilon": epsilon}
+    bn_act_params = {"activation": activation, "use_evo_norm": use_evo_norm, "evo_norm_group_size": evo_norm_group_size, "epsilon": epsilon}
     if preact:  # ResNetV2
-        pre_inputs = batchnorm_with_activation(inputs, activation=activation, zero_gamma=False, name=name + "preact_")
+        pre_inputs = batchnorm_with_activation(inputs, zero_gamma=False, **bn_act_params, name=name + "preact_")
     else:
         pre_inputs = inputs
 
     if conv_shortcut:  # Set a new shortcut using conv
         # short_act = activation if attn_block_params["attn_type"] == "bot" else None
-        shortcut = conv_shortcut_branch(pre_inputs, filters, preact, strides, shortcut_type, None, name=name)
+        shortcut = conv_shortcut_branch(pre_inputs, filters, preact, strides, shortcut_type, bn_params, name=name)  # activation=None
     else:
         shortcut = keras.layers.MaxPooling2D(strides, strides=strides, padding="SAME")(inputs) if strides > 1 else inputs
 
-    deep = deep_branch(pre_inputs, filters, strides, hidden_channel_ratio, use_3x3_kernel, bn_after_attn, activation, attn_block_params, name=name)
+    deep = deep_branch(pre_inputs, filters, strides, hidden_channel_ratio, use_3x3_kernel, bn_after_attn, bn_act_params, attn_block_params, name=name)
 
     # print(f">>>> {inputs.shape = }, {shortcut if shortcut is None else shortcut.shape = }, {deep.shape = }, {filters = }, {strides = }")
     if preact:  # ResNetV2
@@ -170,7 +176,7 @@ def aot_block(
         return keras.layers.Add(name=name + "output")([shortcut, deep]) if shortcut is not None else deep  # if no shortcut
     else:
         if not (use_3x3_kernel and bn_after_attn):
-            deep = batchnorm_with_activation(deep, activation=None, zero_gamma=True, name=name + "3_")
+            deep = batchnorm_with_activation(deep, activation=None, zero_gamma=True, **bn_params, name=name + "3_")
         deep = drop_block(deep, drop_rate)
         out = keras.layers.Add(name=name + "add")([shortcut, deep]) if shortcut is not None else deep  # if no shortcut
         if use_block_output_activation:
@@ -218,15 +224,49 @@ def aot_stack(
     return nn
 
 
-def aot_stem(inputs, stem_width, stem_type=None, activation="relu", quad_stem_act=False, last_strides=1, name=None):
-    """ stem_type in value `[None, "deep", "quad", "tiered"]`. """
-    if stem_type == "deep":
-        nn = deep_stem(inputs, stem_width, activation=activation, last_strides=last_strides, name=name)
-    elif stem_type == "quad":
-        nn = quad_stem(inputs, stem_width, activation=activation, stem_act=quad_stem_act, last_strides=last_strides, name=name)
-    elif stem_type == "tiered":
-        nn = tiered_stem(inputs, stem_width, activation=activation, last_strides=last_strides, name=name)
-    elif stem_type == "kernel_3x3":
+def deep_stem(inputs, stem_width, use_half_channel=True, activation="relu", last_strides=1, bn_params={}, name=None):
+    hidden_channel = stem_width // 2 if use_half_channel else stem_width
+    nn = conv2d_no_bias(inputs, hidden_channel, 3, strides=2, padding="same", name=name and name + "1_")
+    nn = batchnorm_with_activation(nn, activation=activation, **bn_params, name=name and name + "1_")
+    nn = conv2d_no_bias(nn, hidden_channel, 3, strides=1, padding="same", name=name and name + "2_")
+    nn = batchnorm_with_activation(nn, activation=activation, **bn_params, name=name and name + "2_")
+    nn = conv2d_no_bias(nn, stem_width, 3, strides=last_strides, padding="same", name=name and name + "3_")
+    return nn
+
+
+def quad_stem(inputs, stem_width, activation="relu", stem_act=False, last_strides=2, bn_params={}, name=None):
+    nn = conv2d_no_bias(inputs, stem_width // 8, 3, strides=2, padding="same", name=name and name + "1_")
+    if stem_act:
+        nn = batchnorm_with_activation(nn, activation=activation, **bn_params, name=name and name + "1_")
+    nn = conv2d_no_bias(nn, stem_width // 4, 3, strides=1, padding="same", name=name and name + "2_")
+    if stem_act:
+        nn = batchnorm_with_activation(nn, activation=activation, **bn_params, name=name and name + "2_")
+    nn = conv2d_no_bias(nn, stem_width // 2, 3, strides=1, padding="same", name=name and name + "3_")
+    nn = batchnorm_with_activation(nn, activation=activation, **bn_params, name=name and name + "3_")
+    nn = conv2d_no_bias(nn, stem_width, 3, strides=last_strides, padding="same", name=name and name + "4_")
+    return nn
+
+
+def tiered_stem(inputs, stem_width, activation="relu", last_strides=1, bn_params={}, name=None):
+    nn = conv2d_no_bias(inputs, 3 * stem_width // 8, 3, strides=2, padding="same", name=name and name + "1_")
+    nn = batchnorm_with_activation(nn, activation=activation, **bn_params, name=name and name + "1_")
+    nn = conv2d_no_bias(nn, stem_width // 2, 3, strides=1, padding="same", name=name and name + "2_")
+    nn = batchnorm_with_activation(nn, activation=activation, **bn_params, name=name and name + "2_")
+    nn = conv2d_no_bias(nn, stem_width, 3, strides=last_strides, padding="same", name=name and name + "3_")
+    return nn
+
+
+def aot_stem(inputs, stem_width, type=None, activation="relu", quad_stem_act=False, last_strides=1, bn_params={}, name=None):
+    """ stem_type in value `[None, "deep", "deep2", "quad", "tiered"]`. """
+    if type == "deep":
+        nn = deep_stem(inputs, stem_width, activation=activation, last_strides=last_strides, bn_params=bn_params, name=name)
+    elif type == "deep2":  # RegNetZD8_EVO
+        nn = deep_stem(inputs, stem_width, use_half_channel=False, activation=activation, last_strides=last_strides, bn_params=bn_params, name=name)
+    elif type == "quad":
+        nn = quad_stem(inputs, stem_width, activation=activation, stem_act=quad_stem_act, last_strides=last_strides, bn_params=bn_params, name=name)
+    elif type == "tiered":
+        nn = tiered_stem(inputs, stem_width, activation=activation, last_strides=last_strides, bn_params=bn_params, name=name)
+    elif type == "kernel_3x3":
         nn = conv2d_no_bias(inputs, stem_width, 3, strides=2, padding="same", name=name)
     else:
         nn = conv2d_no_bias(inputs, stem_width, 7, strides=2, padding="same", name=name)
@@ -242,6 +282,9 @@ def AotNet(
     hidden_channel_ratio=0.25,
     use_3x3_kernel=False,
     use_block_output_activation=True,
+    use_evo_norm=False,
+    bn_epsilon=1e-5,
+    evo_norm_group_size=-1,
     stem_width=64,  # Stem params
     stem_type=None,  # ["deep", "quad", "tiered", "kernel_3x3", None]
     quad_stem_act=False,
@@ -266,21 +309,27 @@ def AotNet(
     pretrained=None,
     kwargs=None,
 ):
+    """ Stem """
     inputs = keras.layers.Input(shape=input_shape)
-    nn = aot_stem(inputs, stem_width, stem_type, activation, quad_stem_act, last_strides=stem_last_strides, name="stem_")
+    bn_params = {"use_evo_norm": use_evo_norm, "evo_norm_group_size": evo_norm_group_size, "epsilon": bn_epsilon}
+    nn = aot_stem(inputs, stem_width, stem_type, activation, quad_stem_act, last_strides=stem_last_strides, bn_params=bn_params, name="stem_")
 
     if not preact:
-        nn = batchnorm_with_activation(nn, activation=activation, name="stem_")
+        nn = batchnorm_with_activation(nn, activation=activation, **bn_params, name="stem_")
     if stem_downsample:
         nn = keras.layers.ZeroPadding2D(padding=1, name="stem_pool_pad")(nn)
         nn = keras.layers.MaxPooling2D(pool_size=3, strides=2, name="stem_pool")(nn)
 
+    """ Stacks """
     block_params = {  # params same for all blocks
         "preact": preact,
         "use_3x3_kernel": use_3x3_kernel,
         "use_block_output_activation": use_block_output_activation,
         "bn_after_attn": bn_after_attn,
         "shortcut_type": shortcut_type,
+        "use_evo_norm": use_evo_norm,
+        "evo_norm_group_size": evo_norm_group_size,
+        "epsilon": bn_epsilon,
         "activation": activation,
     }
 
@@ -299,9 +348,20 @@ def AotNet(
         nn = aot_stack(nn, num_block, out_channel, stride, strides_first, cur_ratio, drop_connect, block_params, **cur_attn_params, name=name)
 
     if preact:  # resnetv2 like
-        nn = batchnorm_with_activation(nn, activation=activation, zero_gamma=False, name="post_")
+        nn = batchnorm_with_activation(nn, activation=activation, zero_gamma=False, **bn_params, name="post_")
+
+    """ Output """
+    # nn = output_block(nn, output_num_features, activation, num_classes, drop_rate=dropout, classifier_activation=classifier_activation)
+    if output_num_features > 0:
+        nn = conv2d_no_bias(nn, output_num_features, 1, strides=1, name="features_")
+        nn = batchnorm_with_activation(nn, activation=activation, **bn_params, name="features_")
+
+    if num_classes > 0:
+        nn = keras.layers.GlobalAveragePooling2D(name="avg_pool")(nn)
+        if dropout > 0:
+            nn = keras.layers.Dropout(dropout, name="head_drop")(nn)
+        nn = keras.layers.Dense(num_classes, dtype="float32", activation=classifier_activation, name="predictions")(nn)
 
-    nn = output_block(nn, output_num_features, activation, num_classes, drop_rate=dropout, classifier_activation=classifier_activation)
     model = keras.models.Model(inputs, nn, name=model_name)
     add_pre_post_process(model, rescale_mode="torch")
     # reload_model_weights(model, pretrained_dict={}, sub_release="aotnet", input_shape=input_shape, pretrained=pretrained)
