@@ -90,11 +90,16 @@ class MultiHeadRelativePositionalEmbedding(keras.layers.Layer):
         else:
             source_tt = source_layer.relative_position_bias_table  # layer
         # self.relative_position_bias_table.assign(tf.transpose(source_tt))
-        hh = ww = int(tf.math.sqrt(float(source_tt.shape[1] - self.cls_token_pos_len)))
+        hh = ww = int(tf.math.sqrt(float(source_tt.shape[1] - self.cls_token_pos_len)))  # assume source weights are all square shape
         num_heads = source_tt.shape[0]
         ss = tf.reshape(source_tt[:, : hh * ww], (num_heads, hh, ww))  # [num_heads, hh, ww]
         ss = tf.transpose(ss, [1, 2, 0])  # [hh, ww, num_heads]
-        target_hh = target_ww = int(tf.math.sqrt(float(self.relative_position_bias_table.shape[1] - self.cls_token_pos_len)))
+
+        if self.attn_height == -1:
+            target_hh = target_ww = int(tf.math.sqrt(float(self.relative_position_bias_table.shape[1] - self.cls_token_pos_len)))
+        else:
+            target_hh = 2 * self.attn_height - 1
+            target_ww = int(float(self.relative_position_bias_table.shape[1] - self.cls_token_pos_len) / target_hh)
         tt = tf.image.resize(ss, [target_hh, target_ww], method=method)  # [target_hh, target_ww, num_heads]
         tt = tf.reshape(tt, (tt.shape[0] * tt.shape[1], num_heads))  # [target_hh * target_ww, num_heads]
         tt = tf.transpose(tt)  # [num_heads, target_hh * target_ww]
