@@ -131,15 +131,19 @@ def get_image_aspect_aware_random_scale_crop(source_shape, target_shape, scale_m
     return random_image_scale, random_offset_y, random_offset_x
 
 
-def aspect_aware_resize_and_crop_image(image, target_shape, scale=-1, crop_y=0, crop_x=0, method="bilinear", antialias=False):
+def aspect_aware_resize_and_crop_image(image, target_shape, scale=-1, crop_y=0, crop_x=0, letterbox_pad=-1, method="bilinear", antialias=False):
+    letterbox_target_shape = (target_shape[0] - letterbox_pad, target_shape[1] - letterbox_pad) if letterbox_pad > 0 else target_shape
     height, width = tf.cast(tf.shape(image)[0], "float32"), tf.cast(tf.shape(image)[1], "float32")
     if scale == -1:
-        scale = tf.minimum(target_shape[0] / height, target_shape[1] / width)
+        scale = tf.minimum(letterbox_target_shape[0] / height, letterbox_target_shape[1] / width)
     scaled_hh, scaled_ww = int(height * scale), int(width * scale)
     image = tf.image.resize(image, [scaled_hh, scaled_ww], method=method, antialias=antialias)
-    image = image[crop_y : crop_y + target_shape[0], crop_x : crop_x + target_shape[1]]
-    image = tf.image.pad_to_bounding_box(image, 0, 0, target_shape[0], target_shape[1])
-    return image, scale
+    image = image[crop_y : crop_y + letterbox_target_shape[0], crop_x : crop_x + letterbox_target_shape[1]]
+    cropped_shape = tf.shape(image)
+
+    pad_top, pad_left = ((target_shape[0] - cropped_shape[0]) // 2, (target_shape[1] - cropped_shape[1]) // 2) if letterbox_pad >= 0 else (0, 0)
+    image = tf.image.pad_to_bounding_box(image, pad_top, pad_left, target_shape[0], target_shape[1])
+    return image, scale, pad_top, pad_left
 
 
 def random_flip_left_right_with_bboxes(image, bboxes, probability=0.5):
@@ -228,7 +232,7 @@ class RandomProcessImageWithBboxes:
         random_crop_mode=0,  # 0 for eval mode, (0, 1) for random crop, 1 for random largest crop, > 1 for random scale
         resize_method="bilinear",
         resize_antialias=False,
-        use_hsv_augment=True,
+        color_augment_method="random_hsv",
         magnitude=0,
         num_layers=2,
         use_color_increasing=True,
@@ -237,22 +241,25 @@ class RandomProcessImageWithBboxes:
         self.max_labels_per_image = max_labels_per_image
         self.target_shape = target_shape if len(target_shape) == 2 else target_shape[:2]
         self.resize_method, self.resize_antialias, self.random_crop_mode, self.magnitude = resize_method, resize_antialias, random_crop_mode, magnitude
-        if magnitude > 0 and use_hsv_augment:
-            self.randaug_wo_pos = random_hsv
-        elif magnitude > 0:
+        if magnitude > 0:
             from keras_cv_attention_models.imagenet import augment
 
-            # TODO: Need to pick color related methods, "Invert" / "Posterize" may not working well here.
-            print(">>>> RandAugment: magnitude = %d" % magnitude)
-            self.randaug_wo_pos = augment.RandAugment(
-                num_layers=num_layers,
-                magnitude=magnitude,
-                use_cutout=False,
-                use_color_increasing=use_color_increasing,
-                use_positional_related_ops=False,  # Set False to exlude [shear, rotate, translate]
-                **randaug_kwargs,
-            )
-            # self.randaug_wo_pos = random_hsv
+            if color_augment_method.lower() == "autoaug":
+                self.randaug_wo_pos = augment.AutoAugment(augmentation_name="simple")
+            elif color_augment_method.lower() == "randaug":
+                # TODO: Need to pick color related methods, "Invert" / "Posterize" may not working well here.
+                print(">>>> RandAugment: magnitude = %d" % magnitude)
+                self.randaug_wo_pos = augment.RandAugment(
+                    num_layers=num_layers,
+                    magnitude=magnitude,
+                    use_cutout=False,
+                    use_color_increasing=use_color_increasing,
+                    use_positional_related_ops=False,  # Set False to exlude [shear, rotate, translate]
+                    **randaug_kwargs,
+                )
+            else:
+                self.randaug_wo_pos = random_hsv
+
             # RandAugment positional related ops. Including [shear, rotate, translate], Also returns affine transform matrix
             # self.pos_randaug = augment.PositionalRandAugment(num_layers=num_layers, magnitude=magnitude, **randaug_kwargs)
 
@@ -260,7 +267,7 @@ class RandomProcessImageWithBboxes:
         image = datapoint["image"]
         objects = datapoint["objects"]
         bbox, label, is_crowd = tf.cast(objects["bbox"], tf.float32), objects["label"], objects.get("is_crowd", None)
-        if len(image.shape) < 3:
+        if len(image.shape) < 2:
             image = tf_imread(image)
         if is_crowd is not None:
             is_not_crowd = is_crowd == False
@@ -272,7 +279,7 @@ class RandomProcessImageWithBboxes:
 
         if self.random_crop_mode > 1:
             scale, crop_hh, crop_ww = get_image_aspect_aware_random_scale_crop((height, width), self.target_shape)
-            image, scale = aspect_aware_resize_and_crop_image(
+            image, scale, _, _ = aspect_aware_resize_and_crop_image(
                 image, self.target_shape, scale, crop_hh, crop_ww, method=self.resize_method, antialias=self.resize_antialias
             )
         elif self.random_crop_mode == 1:
@@ -282,7 +289,7 @@ class RandomProcessImageWithBboxes:
                 image, self.target_shape, scale=(self.random_crop_mode, 1.0), method=self.resize_method, antialias=self.resize_antialias
             )
         else:
-            image, scale = aspect_aware_resize_and_crop_image(image, self.target_shape, method=self.resize_method, antialias=self.resize_antialias)
+            image, scale, _, _ = aspect_aware_resize_and_crop_image(image, self.target_shape, method=self.resize_method, antialias=self.resize_antialias)
             crop_hh, crop_ww = 0, 0
         bbox = resize_and_crop_bboxes(bbox, (height, width), self.target_shape, scale=scale, offset_y=crop_hh, offset_x=crop_ww)
         bbox, label = refine_bboxes_labels_single(bbox, label)
@@ -309,10 +316,12 @@ class RandomProcessImageWithBboxes:
 
 
 class PositionalRandAugmentWithBboxes:
-    def __init__(self, magnitude=0, num_layers=2, max_labels_per_image=100, **randaug_kwargs):
+    def __init__(self, magnitude=0, num_layers=2, max_labels_per_image=100, positional_augment_methods="rts", **randaug_kwargs):
         from keras_cv_attention_models.imagenet import augment
 
-        self.pos_randaug = augment.PositionalRandAugment(num_layers=num_layers, magnitude=magnitude, **randaug_kwargs)
+        self.pos_randaug = augment.PositionalRandAugment(
+            num_layers=num_layers, magnitude=magnitude, positional_augment_methods=positional_augment_methods, **randaug_kwargs
+        )
         self.max_labels_per_image = max_labels_per_image
 
     def __call_single__(self, inputs):
@@ -367,7 +376,7 @@ def __yolor_bboxes_labels_batch_func__(bboxes, labels, anchor_ratios, feature_si
     return tf.map_fn(bbox_process, bbox_labels)
 
 
-def detection_dataset_from_custom_json(data_path):
+def detection_dataset_from_custom_json(data_path, with_info=False):
     import json
 
     with open(data_path, "r") as ff:
@@ -386,7 +395,7 @@ def detection_dataset_from_custom_json(data_path):
     train_ds = train_ds.apply(tf.data.experimental.assert_cardinality(len(train))).with_options(options)
     test_ds = test_ds.apply(tf.data.experimental.assert_cardinality(len(test))).with_options(options)
     dataset = {"train": train_ds, test_key: test_ds}
-    return dataset, total_images, num_classes
+    return (dataset, total_images, num_classes) if with_info else dataset
 
 
 def init_dataset(
@@ -405,7 +414,8 @@ def init_dataset(
     mosaic_mix_prob=0.0,
     resize_method="bilinear",  # ["bilinear", "bicubic"]
     resize_antialias=False,
-    use_hsv_augment=True,  # Use hsv augment instead of randaug color related. Generally using in other training frameworks.
+    color_augment_method="random_hsv",  # one of ["random_hsv", "autoaug", "randaug"]
+    positional_augment_methods="rts",  # Positional augment method besides scale, combine of r: rotate, t: transplate, s: shear
     magnitude=0,
     num_layers=2,
     seed=None,
@@ -415,7 +425,7 @@ def init_dataset(
     # anchor_num_scales=3,  # Force using 1 if use_anchor_free_mode
     # anchor_grid_zero_start="auto",  # False for anchor_free_mode, True for others.
     if data_name.endswith(".json"):
-        dataset, total_images, num_classes = detection_dataset_from_custom_json(data_name)
+        dataset, total_images, num_classes = detection_dataset_from_custom_json(data_name, with_info=True)
     else:
         try_gcs = True if len(tf.config.list_logical_devices("TPU")) > 0 else False
         dataset, info = tfds.load(data_name, with_info=True, try_gcs=try_gcs)
@@ -433,7 +443,7 @@ def init_dataset(
         random_crop_mode=random_crop_mode,
         resize_method=resize_method,
         resize_antialias=resize_antialias,
-        use_hsv_augment=use_hsv_augment,
+        color_augment_method=color_augment_method,
         magnitude=magnitude,
         num_layers=num_layers,
         **augment_kwargs,
@@ -453,7 +463,8 @@ def init_dataset(
     if magnitude > 0:
         # Apply randaug rotate / shear / transform after mosaic mix
         max_labels_per_image = (max_labels_per_image * 4) if mosaic_mix_prob > 0 else max_labels_per_image
-        pos_aug = PositionalRandAugmentWithBboxes(magnitude, num_layers, max_labels_per_image, **augment_kwargs)
+        pos_aug = PositionalRandAugmentWithBboxes(magnitude, num_layers, max_labels_per_image, positional_augment_methods, **augment_kwargs)
+        print(">>>> positional augment methods:", pos_aug.pos_randaug.available_ops)
         train_dataset = train_dataset.map(pos_aug, num_parallel_calls=AUTOTUNE)
 
     if use_anchor_free_mode:
