@@ -204,7 +204,7 @@ class RandomProcessImage:
     def __call__(self, datapoint):
         image = datapoint["image"]
         # image = tf.cond(tf.rank(image) > 0, lambda: image, lambda: tf_imread(image))
-        if len(image.shape) < 3:
+        if len(image.shape) < 2:
             image = tf_imread(image)
         if self.random_crop_min > 0 and self.random_crop_min < 1:
             hh, ww = random_crop_fraction(tf.shape(image), scale=(self.random_crop_min, 1.0))
@@ -255,7 +255,7 @@ def evaluation_process_resize_crop(datapoint, target_shape=(224, 224), central_c
     return image, label
 
 
-def recognition_dataset_from_custom_json(data_path):
+def recognition_dataset_from_custom_json(data_path, with_info=False):
     import json
 
     with open(data_path, "r") as ff:
@@ -273,7 +273,7 @@ def recognition_dataset_from_custom_json(data_path):
     train_ds = train_ds.apply(tf.data.experimental.assert_cardinality(len(train))).with_options(options)
     test_ds = test_ds.apply(tf.data.experimental.assert_cardinality(len(test))).with_options(options)
     dataset = {"train": train_ds, test_key: test_ds}
-    return dataset, total_images, num_classes
+    return (dataset, total_images, num_classes) if with_info else dataset
 
 
 def init_dataset(
@@ -296,7 +296,7 @@ def init_dataset(
 ):
     # print(">>>> Dataset args:", locals())
     if data_name.endswith(".json"):
-        dataset, total_images, num_classes = recognition_dataset_from_custom_json(data_name)
+        dataset, total_images, num_classes = recognition_dataset_from_custom_json(data_name, with_info=True)
     else:
         try_gcs = True if len(tf.config.list_logical_devices("TPU")) > 0 else False
         dataset, info = tfds.load(data_name, with_info=True, try_gcs=try_gcs)
@@ -353,3 +353,26 @@ def init_dataset(
         test_dataset = dataset["test"].map(test_process)
     test_dataset = test_dataset.batch(batch_size).map(lambda xx, yy: (rescaling(xx), as_one_hot(yy)))
     return train_dataset, test_dataset, total_images, num_classes, steps_per_epoch
+
+
+""" Show """
+
+
+def show_batch_sample(dataset, rescale_mode="tf", rows=-1, base_size=3):
+    import matplotlib.pyplot as plt
+    from keras_cv_attention_models.visualizing import get_plot_cols_rows, stack_and_plot_images
+
+    if isinstance(dataset, (list, tuple)):
+        images, labels = dataset
+    else:
+        images, labels = dataset.as_numpy_iterator().next()
+    mean, std = init_mean_std_by_rescale_mode(rescale_mode)
+    mean, std = (mean.numpy(), std.numpy()) if hasattr(mean, "numpy") else (mean, std)
+    images = (images * std + mean) / 255
+
+    if labels.shape[-1] == 1000:
+        labels = [ii[0][1] for ii in keras.applications.imagenet_utils.decode_predictions(labels, top=1)]
+    else:
+        labels = tf.argmax(labels, axis=-1).numpy()
+    ax, _ = stack_and_plot_images(images, texts=labels, rows=rows, ax=None, base_size=base_size)
+    return ax
