@@ -59,14 +59,14 @@ YOLOX.__doc__ = __head_doc__.format(depth_mul=1, width_mul=1) + """  depth_mul:
   model_name: string, model name.
 """ + __tail_doc__ + """
 Model architectures:
-  | Model     | Params | Image resolution | COCO test AP |
-  | --------- | ------ | ---------------- | ------------ |
-  | YOLOXNano | 0.91M  | 416              | 25.8         |
-  | YOLOXTiny | 5.06M  | 416              | 32.8         |
-  | YOLOXS    | 9.0M   | 640              | 40.5         |
-  | YOLOXM    | 25.3M  | 640              | 47.2         |
-  | YOLOXL    | 54.2M  | 640              | 50.1         |
-  | YOLOXX    | 99.1M  | 640              | 51.5         |
+  | Model     | Params | Image resolution | COCO val AP | test AP |
+  | --------- | ------ | ---------------- | ----------- | ------- |
+  | YOLOXNano | 0.91M  | 416              | 25.8        |         |
+  | YOLOXTiny | 5.06M  | 416              | 32.8        |         |
+  | YOLOXS    | 9.0M   | 640              | 40.5        | 40.5    |
+  | YOLOXM    | 25.3M  | 640              | 46.9        | 47.2    |
+  | YOLOXL    | 54.2M  | 640              | 49.7        | 50.1    |
+  | YOLOXX    | 99.1M  | 640              | 51.5        | 51.5    |
 """
 
 YOLOXNano.__doc__ = __head_doc__.format(depth_mul=0.33, width_mul=0.25) + __tail_doc__
